@@ -101,6 +101,11 @@ const TestInfo tests[] = {
 	{ "functions/functions-xpath-7", "functions/data-1", FALSE },
 	{ "functions/functions-xpath-8", "functions/data-1", FALSE },
 	{ "functions/functions-xpath-9", "functions/data-1", FALSE },
+	{ "functions/functions-xpath-10", "functions/data-4", FALSE },
+	{ "functions/functions-xpath-11", "functions/data-4", FALSE },
+	{ "functions/functions-xpath-12", "functions/data-4", FALSE },
+	{ "functions/functions-xpath-13", "functions/data-4", FALSE },
+	{ "functions/functions-xpath-14", "functions/data-4", FALSE },
 	{ "graph/graph-1", "graph/data-1", FALSE },
 	{ "graph/graph-2", "graph/data-2", FALSE },
 	{ "graph/graph-3", "graph/data-3", FALSE },
@@ -132,6 +137,11 @@ const TestInfo tests[] = {
 	/* Mixed cardinality tests */
 	{ "mixed-cardinality/insert-mixed-cardinality-query-1", "mixed-cardinality/insert-mixed-cardinality-1", FALSE, FALSE },
 	{ "mixed-cardinality/update-mixed-cardinality-query-1", "mixed-cardinality/update-mixed-cardinality-1", FALSE, FALSE },
+	/* Bind tests */
+	{ "bind/bind1", "bind/data", FALSE },
+	{ "bind/bind2", "bind/data", FALSE },
+	{ "bind/bind3", "bind/data", FALSE },
+	{ "bind/bind4", "bind/data", FALSE },
 	{ NULL }
 };
 
